/* ELECTRONOOBS 2023
 * NOTE: Run the ATtiny85 at 1-MHz internal clock
 * This requires "Burn Bootloader" to be run first
 * The "0" bit: 188us
 * The "1" bit: 3x zero bit length
 * Sync (start) time: 30x zero bit
 * Message length: 24 data bits, 1 stop bit*/

//Inputs/Outputs
#define Pulse_In 3              // Pin from the radio receiver part
#define RX_LED 4                // An LED connected on pin 4 for warning messages
#define SSR 0                   // The SSR relay circuit is connected on this pin
#define Msg_Length 3            // Defined message length for specific transmitter


//Variables used in the code
byte RF_Bit_Count;
byte RF_Byte_Count;
byte RF_Byte = 0;
byte RF_Message[] = {0, 0, 0};  // 24 bits
unsigned long Start_Time = 0;
int Pulse_Width;
byte Started = false;
bool light_state = false;

// All times in microseconds
#define Min_Gap_Time 5000       // nominal = 5600
#define Max_Gap_Time 6200       // nominal = 5600
#define One_Bit_Time 120        // nominal = 188
#define Zero_Bit_Time 500       // nominal = 564

// Defines for expected on/off commands
#define RF_code_1 0x33
#define RF_code_2 0x45
#define RF_code_3 0x15 


void setup() {  
  pinMode(SSR, OUTPUT);
  digitalWrite(SSR, HIGH);
  pinMode(RX_LED, OUTPUT);
  digitalWrite(RX_LED, LOW);
  digitalWrite(Pulse_In, HIGH);       //Set the RX pin with a pullup 

  // Set interrupt-on-change registers
  bitClear(GIMSK, INT0);              // disable INT0 external interrupt
  bitSet(GIMSK, PCIE);                // enable interrupt-on-change
  bitSet(PCMSK, Pulse_In);
}

void loop(){
  byte i;
  if (RF_Byte_Count == Msg_Length) {
    noInterrupts();

    if ((RF_Message[0] == RF_code_1) && (RF_Message[1] == RF_code_2) && (RF_Message[2] == RF_code_3)){
      light_state = !light_state;
      RF_Message[0] = 0;            //Reset RF_Message value
      RF_Message[1] = 0; 
      RF_Message[2] = 0;   
      if(light_state){
        digitalWrite(RX_LED, 1);   
        digitalWrite(SSR, LOW);     //Turn ON SSR (PNP INVERTED)
      }
      else if (!light_state){
        digitalWrite(RX_LED, 0);
        digitalWrite(SSR, HIGH);    //Turn OFF SSR (PNP INVERTED)
      }
      delay(400);
    }

    
    Started = false;
    interrupts();
  }
} // end loop




// Interrupt-on-change handler
ISR (PCINT0_vect)
{
  //when the pin goes LOW record the pulse start time
  if (digitalRead(Pulse_In) == LOW) {
    Start_Time = micros();
  }
  else // pin went high
    if (Start_Time != 0) {
      //calculate the pulse width
      Pulse_Width = ((volatile int)micros() - Start_Time);
      //clear the timer
      Start_Time = 0;

      // Now check for valid message
      if (Pulse_Width < Max_Gap_Time) {
        if (Started == true) {
          if (Pulse_Width > Zero_Bit_Time) {
            // bit = 0
            RF_Bit_Count--; // bits received MSB first
            bitClear(RF_Message[RF_Byte_Count], RF_Bit_Count);
            if (RF_Bit_Count == 0) {
              RF_Byte_Count++;
              RF_Bit_Count = 8;
            }
          }
          else if (Pulse_Width > One_Bit_Time) {
            // bit = 1
            RF_Bit_Count--; // bits received MSB first
            bitSet(RF_Message[RF_Byte_Count], RF_Bit_Count);
            if (RF_Bit_Count == 0) {
              RF_Byte_Count++;
              RF_Bit_Count = 8;
            }
          }
          else { // bad bit
            Started = false;
          }
        } // Started
        
        else if (Pulse_Width > Min_Gap_Time) {
          // start of message
          Started = true;
          RF_Byte_Count = 0;
          RF_Bit_Count = 8;
        }
      }
      else { // invalid message
        Started = false;
      }
    }
}
